# -*- encoding: utf-8 -*-

import os
import time
import random
import hashlib
import pkg_resources
from paste.script import templates
try:
    random = random.SystemRandom()
    using_sysrandom = True
except NotImplementedError:
    import warnings
    warnings.warn('A secure pseudo-random number generator is not available '
                  'on your system. Falling back to Mersenne Twister.')
    using_sysrandom = False

RENOMMAGES = {'gitignore': '.gitignore'}


# Taken directly from django's utils/crypto.py, removed
# settings.SECRET_KEY reference.
def get_random_string(length=12,
                      allowed_chars='abcdefghijklmnopqrstuvwxyz'
                                    'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'):
    """
    Returns a securely generated random string.

    The default length of 12 with the a-z, A-Z, 0-9 character set returns
    a 71-bit value. log_2((26+26+10)^12) =~ 71 bits
    """
    if not using_sysrandom:
        # This is ugly, and a hack, but it makes things better than
        # the alternative of predictability. This re-seeds the PRNG
        # using a value that is hard for an attacker to predict, every
        # time a random string is required. This may change the
        # properties of the chosen random sequence slightly, but this
        # is better than absolute predictability.
        random.seed(
            hashlib.sha256(
                ("%s%s%s" % (
                    random.getstate(),
                    time.time(),
                    '')).encode('utf-8')
                ).digest())
    return ''.join([random.choice(allowed_chars) for i in range(length)])


class DjangoTemplate(templates.Template):
    """
    Modèle de projet Django pour l'AUF reposant sur
    `Buildout <http://www.buildout.org/>`_.

    Il permet de construire un projet avec les briques de base
    importantes pour la maintenance et l'homogénéité entre les applications.

    **Utilisation**::

        paster create -t auf_django


    **Les fichiers squelettes**:

    *buildout.cfg*

    .. literalinclude:: ../auf/skeleton/django/templates/django/\
            buildout.cfg_tmpl

    *devel.cfg*

    .. literalinclude:: ../auf/skeleton/django/templates/django/\
            devel.cfg_tmpl

    *versions.cfg*

    .. literalinclude:: ../auf/skeleton/django/templates/django/\
            versions.cfg_tmpl

    """
    egg_plugins = ['auf.skeleton.django']
    summary = 'Démarrer un nouveau projet Django avec Buildout'
    use_cheetah = True

    vars = [
        templates.var('module_name', u'nom du module projet?',
        default='project'),
        templates.var('django_version', u'Version de Django?',),
    ]

    def post(self, command, output_dir, vars):
        for avant, apres in RENOMMAGES.items():
            avant = os.path.join(output_dir, avant)
            apres = os.path.join(output_dir, apres)
            os.rename(avant, apres)
        print ""
        print "Génération du projet terminée."
        print ""
        print "* Paquets requis"
        print "- python-virtualenv"
        print "- python-mysqldb"
        print "- git-core"
        print ""
        print ""
        print "* : cd %s" % output_dir
        print "* : python bootstrap.py"
        print "* : bin/buildout -c devel.cfg"
        print "* : Configurer conf.py"
        print "* : bin/django syncdb"
        print "* : bin/django migrate"
        print "* : git push origin <depot>"


class Django13Template(DjangoTemplate):
    _template_dir = 'templates/django_1_3'


class Django145Template(DjangoTemplate):
    def post(self, command, output_dir, vars):
        super(Django145Template, self).post(command, output_dir, vars)

        # Copier conf.py par défaut
        if vars['copy_conf_py'] == 'o':
            avant = open(
                os.path.join(output_dir,
                        '%(module_name)s/conf.py.edit' % vars), 'r'
                             )
            apres = open(
                os.path.join(output_dir,
                        '%(module_name)s/conf.py' % vars), 'w')
            apres.write(avant.read())
            avant.close()
            apres.close()
            
    vars = [
        templates.var('module_name', u'nom du module projet?',
        default='project'),
        templates.var('site_id',
                      'ID de site',
                      default='1'),
        templates.var('secret_key',
                      'Clé secrète',
                      default=get_random_string(50),
                      ),
        templates.var('copy_conf_py',
                      'Copier le conf.py par défaut? (o/n)',
                      default='o',
                      ),
        ]

    _template_dir = 'templates/django-1.4.5'
